; Auf einen Wert ungleich 0 setzen, wenn das Programm
; auf einem echten Prozessor ausgefhrt werden soll:
.equ RealCPU, 0x00

.equ StepCount ,0x50
.equ T0Low     ,0x51
.equ T0High    ,0x52
.equ T1Low     ,0x53
.equ T1High    ,0x54

.org 0x100
  mov StepCount,#0x00

  ; Port initialisieren
  mov P1,#0xFF

  ;Stift in eine beliebige Ausgangsposition bewegen
  lcall SerialMove

  ; beide Timer in Mode 1 = 16-Bit Timer
  mov 0x89,#0x11 ; TMOD = 0x89

  ; Timer einschalten und Interrupts freischalten
  mov 0x88,#0x50 ; TCON = 0x88
  mov 0xA8,#0x8A ; IE = 0xA8

Hauptschleife:

  ;Daten von der seriellen Schnittstelle lesen
  lcall ReadData
  jnz Ende

  lcall InitMove

loop:
  mov a, StepCount
  jnz loop

  lcall PrintOK
  sjmp Hauptschleife

Ende:
  ljmp 0x918E



; ================ Stift an Startposition bewegen ======================

.equ ESC, 0x1B
.equ Zwei, 50
.equ Vier, 52
.equ Sechs, 54
.equ Acht, 56

; Stift steuern durch Eingaben ber 
; die serielle Schnittstelle (2,4,6,8)
; ====================================
SerialMove:
  SerialRead: jnb RI,SerialRead
  mov a,SBUF
  clr RI

  cjne a,#ESC,NotEscape
  ret
  NotEscape:
  
  cjne a,#Zwei,NotZwei
  clr P1.3
  cpl P1.1
  sjmp SerialMove
  NotZwei:

  cjne a,#Vier,NotVier
  clr P1.2
  cpl P1.0
  sjmp SerialMove
  NotVier:

  cjne a,#Sechs,NotSechs
  setb P1.2
  cpl P1.0
  sjmp SerialMove
  NotSechs:

  cjne a,#Acht,SerialMove
  setb P1.3
  cpl P1.1
  sjmp SerialMove


; Verzgerung, so da die Schrittfrequenz 
; nicht mehr als 800Hz betrgt.
; =======================================
Warten:
  push ACC

  ; Nur warten, wenn das Programm nicht im Emulator luft
  mov a,#RealCPU
  jnz DoWait
  pop ACC
  ret
  DoWait:

  mov a,r7
  push ACC
  mov a,r6
  push ACC

  ; 800Hz ungefhr = 2083 Taktschritte 
  ; whle: 2560 = 0x0A00 Taktschritte Pause
  mov r7,#0x0A
  mov r6,#0x00
  sjmp WarteSchleife2
  
WarteSchleife1:
  mov r6,#0xFF
WarteSchleife2:
  nop
  nop
  nop
  djnz r6,WarteSchleife2
  djnz r7,WarteSchleife1

  pop ACC  
  mov r7,0
  pop ACC
  mov r6,a

  pop ACC

  ret


; ================ Ansteuerung der seriellen Schnittstelle ===============


; Liet ein Zeichen und speichert es im r0
; ========================================
ReadChar:  
  Lesen: jnb RI,Lesen
  mov r0,SBUF
  clr RI
  ret


; Wandelt ACSII-Zeichen aus r0 in Hex-Wert um und speichert
; diesen in r0. Bei Fehler ist a ungleich 0, sonst gleich 0.
; =======================================================

HalbConvertToHex:
  ; Abbruch, wenn ASCII-Wert kleiner 48 ist:
  mov a,r0
  clr c
  subb a,#48  ; 48d = '0'
  jc HalbConvertFehler

  ; wenn ASCII-Wert kleiner 58, dann ist es eine Zahl.
  mov a,r0
  clr c
  subb a,#58
  jc ZahlToHex

  ; Abbruch, wenn ASCII-Wert grer 70 ist:
  mov a,r0
  clr c
  subb a,#71  ; 70d = 'F'
  jnc HalbConvertFehler

  ; wenn ASCII-Wert grer gleich 65, dann ist es ein Buchstabe.
  mov a,r0
  clr c
  subb a,#65
  jnc ABCToHex

  sjmp HalbConvertFehler

ZahlToHex:
  mov a,r0
  clr c
  subb a,#48
  mov r0,a
  mov a,#0x00
  ret

ABCToHex:
  mov a,r0
  clr c
  subb a,#55
  mov r0,a
  mov a,#0x00
  ret

HalbConvertFehler:
  mov a,#0x01
  ret


; Liest ein ASCII-Zeichen von der seriellen Schnittstelle,
; wandelt es in ein halbes Hex-Byte um und gibt es in r0 zurck.
; Akku=0: Ok
; Akku=1: Abbruch
; Akku=2: Fehler
; ===============================================

ReadHalfByte:
  lcall ReadChar
  cjne r0,#ESC,ReadHalfByteOK
  mov a,#0x01
  ret

  ReadHalfByteOK:
  lcall HalbConvertToHex ; ASCII r0 -> Hex r0
  jnz ConvertFehler
  mov a,#0x00
  ret

  ConvertFehler:
  mov a,#0x01
  ret


; Liest zwei ASCII-Zeichen von der seriellen Schnittstelle,
; wandelt sei in ein Hex-Byte um und gibt es in r0 zurck.
; Akku=0: Ok
; Akku=1: Abbruch
; Akku=2: Fehler
; ===============================================

ReadByte:
  lcall ReadHalfByte
  jnz ReadByteFehler
  mov a,r0
  rl a
  rl a
  rl a
  rl a
  push ACC

  lcall ReadHalfByte
  jnz ReadByteFehler2

  pop ACC
  add a,r0
  mov r0,a
  mov a,#0x00
  ret

  ReadByteFehler2:
  mov r0,a
  pop ACC
  mov a,r0
  ReadByteFehler:

  ret


; ================ Ausgabe von Texten ===============


Meldungsschleife:
  mov a,#0x00  
  movc a,@a+dptr
  MeldungSchreiben: jnb TI,MeldungSchreiben
  clr TI
  mov SBUF,a
  inc dptr
  djnz r2,Meldungsschleife
  ret

PrintFehler:
  mov dptr,#MeldungstextFehler
  mov r2,#8
  sjmp Meldungsschleife
  MeldungstextFehler:
  .word 0x4665, 0x686C, 0x6572, 0x0A0D ; "Fehler"

PrintOK:
  mov dptr,#MeldungstextOK
  mov r2,#4
  sjmp Meldungsschleife
  MeldungstextOK:
  .word 0x4F4B, 0x0A0D ; "OK"


; ============ Einlesen und Bearbeiten der Steuerdaten ===========

ReadData:
  lcall ReadHalfByte
  jnz ReadDataFehler
  mov a,r0
  mov r4,a
  lcall ReadByte  
  jnz ReadDataFehler
  mov a,r0
  mov r5,a
  lcall ReadByte  
  jnz ReadDataFehler
  mov a,r0
  mov r6,a  
  lcall ReadByte  
  jnz ReadDataFehler
  mov a,r0
  mov r7,a

  mov a,#0x00
  ret
  
  ReadDataFehler:
  cjne a,#0x01,ReadDataAbbruch
  lcall PrintFehler
  sjmp ReadData  

  ReadDataAbbruch:
  ret


InitMove:

  ; Richtung setzen
  mov a,r4
  rl a
  rl a
  mov P1,a

  ; r5*250 + 4000
  mov a,r5
  mov B,#0xFA ; = 250d
  mul ab
  add a,#0xA0
  mov T0Low,a
  mov a,B
  addc a,#0x0F
  mov T0High,a

  ; r6*63 + 1000
  mov a,r6
  mov B,#0x3F ; = 63d
  mul ab
  add a,#0xE8
  mov T1Low,a
  mov a,B
  addc a,#0x03
  mov T1High,a

  mov StepCount,r7

  mov TL0,T0Low
  mov TH0,T0High
  mov TL1,T1Low
  mov TH1,T1High
  ret

; Behandlungsroutinen fr die Timer Interrupts
; ============================================

XSchritt:
  mov a,StepCount
  jz Back
  mov TL0,T0Low
  mov TH0,T0High
  dec StepCount
  cpl P1.0
  reti

YSchritt:
  mov a,StepCount
  jz Back
  mov TL1,T1Low
  mov TH1,T1High
  dec StepCount
  cpl P1.1
  reti

Back:
  reti

.org 0x0B
  ljmp XSchritt
.org 0x1B
  ljmp YSchritt 
.end
