.equ ESC,   0x1B

.equ ASCIIDaten, 0x40
.equ HexDaten,   0x48
.equ Ergebnis,   0x40   

.equ Ahigh,      0x48
.equ Alow,       0x49
.equ Bhigh,      0x4A
.equ Blow,       0x4B
.equ Chigh,      0x4C
.equ Clow,       0x4D

.org 0x100
  ; kein Interrupt fr serielle Schnittstelle
  clr ES

Hauptschleife:

  lcall SummandenLesen
  jnz Programmende  

  mov r0,#ASCIIDaten
  mov r1,#HexDaten
  mov r2,#0x04

ConvertSchleife1:
  lcall ConvertToHex
  jz Weiter2
  lcall Fehlermeldung
  ljmp Hauptschleife
  Weiter2:
  djnz r2,ConvertSchleife1

  ; Addieren: C=A+B
  mov a,Alow
  add a,Blow
  mov Clow,a
  mov a,Ahigh
  addc a,Bhigh
  mov Chigh,a

  ; Umwandeln in ASCII-Daten
  mov a,Chigh
  mov r0,#Ergebnis
  lcall ConvertToASCII
  mov a,Clow
  lcall ConvertToASCII

  ; Daten zurckschreiben
  mov r0,#Ergebnis
  mov r1,#0x04
  Schreibeschleife:
  Schreiben: jnb TI,Schreiben
  clr TI
  mov SBUF,@r0
  inc r0
  djnz r1,Schreibeschleife

  ljmp Hauptschleife

Programmende:
  setb ES
  ljmp 0x918E


; =======================================================
; Liet die beiden Summanden (jeweils 4 Zeichen)
; von der seriellen Schnittstelle, und speichert
; sie ab Adresse "ASCIIDaten". Wird statt eines
; Zeichens "Escape" geschickt, so bricht das
; Unterprogramm ab und gibt a=1 zurck, sonst a=0.

SummandenLesen:

  mov r0,#0x08
  mov r1,#ASCIIDaten

LeseSchleife:

  Lesen: jnb RI,Lesen
  mov a,SBUF
  clr RI

  cjne a,#ESC,WeiterLesen
  mov a,#0x01
  ret

  WeiterLesen:
  mov @r1,a
  inc r1
  djnz r0,LeseSchleife
  mov a,#0x00
  ret


; =======================================================
; Wandelt ACSII-Zeichen aus @r0 in Hex-Wert um und speichert
; diesen in r1. Bei Fehler ist a ungleich 0, sonst gleich 0.

HalbConvertToHex:
  ; Abbruch, wenn ASCII-Wert kleiner 48 ist:
  mov a,@r0
  clr c
  subb a,#48  ; 48d = '0'
  jc HalbConvertFehler

  ; wenn ASCII-Wert kleiner 58, dann ist es eine Zahl.
  mov a,@r0
  clr c
  subb a,#58
  jc ZahlToHex

  ; Abbruch, wenn ASCII-Wert grer 70 ist:
  mov a,@r0
  clr c
  subb a,#71  ; 70d = 'F'
  jnc HalbConvertFehler

  ; wenn ASCII-Wert grer gleich 65, dann ist es ein Buchstabe.
  mov a,@r0
  clr c
  subb a,#65
  jnc ABCToHex

  sjmp HalbConvertFehler

ZahlToHex:
  mov a,@r0
  clr c
  subb a,#48
  mov @r1,a
  mov a,#0x00
  ret

ABCToHex:
  mov a,@r0
  clr c
  subb a,#55
  mov @r1,a
  mov a,#0x00
  ret

HalbConvertFehler:
  mov a,#0x01
  ret


; =======================================================
; Wandelt ACSII-Zeichen aus @r0 und @(r0+1) in Hex-Wert um
; und speichert diesen in @r1. Bei Fehler ist a ungleich 0,
; sonst gleich 0. Zustzlich wird r0 um 2 und r1 um 1 erhht.

ConvertToHex:
  ; r2-Register sichern
  mov B,r2
  push B

  lcall HalbConvertToHex ; ASCII @r0 -> Hex @r1 
  jnz ConvertEnde        ; wenn a ungleich 0: Fehler, Ende

  mov a,@r1 ; Ergebnis aus @r1 in das obere Halbbyte von r2
  mov B,#0x10
  mul ab      
  mov r2,a

  inc r0      

  lcall HalbConvertToHex ; ASCII @r0 -> Hex @r1 
  jnz ConvertEnde        ; wenn a ungleich 0: Fehler, Ende

  mov a,r2 ; Ergebnis aus r1 zu r2 addieren (unteres Halbbyte)
  add a,@r1
  mov @r1,a
  mov a,#0x00

  inc r0
  inc r1

  ConvertEnde:
  
  ; r2-Register wiederherstellen
  pop B
  mov r2,B

  ret


; =======================================================
; Gibt eine Fehlermeldung auf der seriellen Schnittstelle aus.

Fehlermeldung:
  mov dptr,#Meldungstext
  mov r2,#7

Meldungsschleife:
  mov a,#0x00  
  movc a,@a+dptr
  MeldungSchreiben: jnb TI,MeldungSchreiben
  clr TI
  mov SBUF,a
  inc dptr
  djnz r2,Meldungsschleife
  ret

  Meldungstext:
  .word 0x4665, 0x686C, 0x6572, 0x0A00 ; "Fehler"


; =======================================================
; Wandelt Hex-Code a zu halbem ASCII-Zeichen in a um.

HalbConvertToASCII:
  push ACC
  clr c
  subb a,#0x0A
  pop ACC
  jc ZahlToASCII
  add a,#55
  ret
ZahlToASCII:
  add a,#48
  ret

; =======================================================
; Byte aus a in ASCII-Code umwandeln und in @r0 und @(r0+1)
; speichern. Danach r0 um 2 erhhen.

ConvertToASCII:
  push ACC
  anl a,#0xF0
  rr a
  rr a
  rr a
  rr a

  lcall HalbConvertToASCII
  mov @r0,a
  inc r0

  pop ACC
  anl a,#0x0F

  lcall HalbConvertToASCII
  mov @r0,a
  inc r0

  ret

.end